@@ -49,11 +49,21 @@ pub trait MathematicalOps {
     /// tolerance of roughly `0.0000002`.
     fn exp(&self) -> Decimal;
 
+    /// The estimated exponential function, e<sup>x</sup>. Stops calculating when it is within
+    /// tolerance of roughly `0.0000002`. Returns `None` on overflow.
+    fn checked_exp(&self) -> Option<Decimal>;
+
     /// The estimated exponential function, e<sup>x</sup> using the `tolerance` provided as a hint
     /// as to when to stop calculating. A larger tolerance will cause the number to stop calculating
     /// sooner at the potential cost of a slightly less accurate result.
     fn exp_with_tolerance(&self, tolerance: Decimal) -> Decimal;
 
+    /// The estimated exponential function, e<sup>x</sup> using the `tolerance` provided as a hint
+    /// as to when to stop calculating. A larger tolerance will cause the number to stop calculating
+    /// sooner at the potential cost of a slightly less accurate result.
+    /// Returns `None` on overflow.
+    fn checked_exp_with_tolerance(&self, tolerance: Decimal) -> Option<Decimal>;
+
     /// Raise self to the given integer exponent: x<sup>y</sup>
     fn powi(&self, exp: i64) -> Decimal;
 
@@ -93,8 +103,11 @@ pub trait MathematicalOps {
     /// The Cumulative distribution function for a Normal distribution
     fn norm_cdf(&self) -> Decimal;
 
-    /// The Probability density function for a Normal distribution
+    /// The Probability density function for a Normal distribution.
     fn norm_pdf(&self) -> Decimal;
+
+    /// The Probability density function for a Normal distribution returning `None` on overflow.
+    fn checked_norm_pdf(&self) -> Option<Decimal>;
 }
 
 impl MathematicalOps for Decimal {
@@ -102,17 +115,28 @@ impl MathematicalOps for Decimal {
         self.exp_with_tolerance(EXP_TOLERANCE)
     }
 
-    #[inline]
+    fn checked_exp(&self) -> Option<Decimal> {
+        self.checked_exp_with_tolerance(EXP_TOLERANCE)
+    }
+
     fn exp_with_tolerance(&self, tolerance: Decimal) -> Decimal {
+        match self.checked_exp_with_tolerance(tolerance) {
+            Some(d) => d,
+            None => panic!("Exp overflowed"),
+        }
+    }
+
+    #[inline]
+    fn checked_exp_with_tolerance(&self, tolerance: Decimal) -> Option<Decimal> {
         if self.is_zero() {
-            return Decimal::ONE;
+            return Some(Decimal::ONE);
         }
 
         let mut term = *self;
         let mut result = self + Decimal::ONE;
 
         for factorial in FACTORIAL.iter().skip(2) {
-            term = self * term;
+            term = self.checked_mul(term)?;
             let next = result + (term / factorial);
             let diff = (next - result).abs();
             result = next;
@@ -121,7 +145,7 @@ impl MathematicalOps for Decimal {
             }
         }
 
-        result
+        Some(result)
     }
 
     fn powi(&self, exp: i64) -> Decimal {
@@ -248,7 +272,7 @@ impl MathematicalOps for Decimal {
             Some(e) => e,
             None => return None,
         };
-        let mut result = e.exp();
+        let mut result = e.checked_exp()?;
         result.set_sign_negative(negative);
         Some(result)
     }
@@ -330,10 +354,20 @@ impl MathematicalOps for Decimal {
         (Decimal::ONE + (self / Decimal::from_parts(2318911239, 3292722, 0, false, 16)).erf()) / TWO
     }
 
-    /// The Probability density function for a Normal distribution
+    /// The Probability density function for a Normal distribution.
     fn norm_pdf(&self) -> Decimal {
+        match self.checked_norm_pdf() {
+            Some(d) => d,
+            None => panic!("Norm Pdf overflowed"),
+        }
+    }
+
+    /// The Probability density function for a Normal distribution, return `None` on overflow.
+    fn checked_norm_pdf(&self) -> Option<Decimal> {
         let sqrt2pi = Decimal::from_parts_raw(2133383024, 2079885984, 1358845910, 1835008);
-        (-self.powi(2) / TWO).exp() / sqrt2pi
+        let factor = -self.checked_powi(2)?;
+        let factor = factor.checked_div(TWO)?;
+        factor.checked_exp()?.checked_div(sqrt2pi)
     }
 }
 
